from django.urls import path

from mcmailer.views import connect_gmail, g_auth_endpoint, send_test_email

app_name = 'mcmailer'
urlpatterns = [
    path('send-test-email/', send_test_email, name='send_test_email'),
    path('connect/', connect_gmail, name='connect_gmail'),
    path('g-auth-endpoint/', g_auth_endpoint, name='g_auth_endpoint'),
]



# Generated by Django 3.1.5 on 2021-04-11 08:11

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EmailNotificationRecipient',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
            ],
            options={
                'verbose_name_plural': 'Email Notification Recipients',
            },
        ),
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('preview_text', models.CharField(max_length=255)),
                ('preview_text_en', models.CharField(max_length=255, null=True)),
                ('preview_text_el', models.CharField(max_length=255, null=True)),
                ('subject', models.CharField(max_length=255)),
                ('subject_en', models.CharField(max_length=255, null=True)),
                ('subject_el', models.CharField(max_length=255, null=True)),
                ('plain_body', models.TextField()),
                ('plain_body_en', models.TextField(null=True)),
                ('plain_body_el', models.TextField(null=True)),
                ('html_body', models.TextField()),
                ('html_body_en', models.TextField(null=True)),
                ('html_body_el', models.TextField(null=True)),
            ],
        ),
        migrations.CreateModel(
            name='GmailConnectionToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('json_string', models.TextField()),
                ('valid', models.BooleanField()),
                ('gmail_address', models.EmailField(max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name='SystemFromEmailAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
                ('from_official_name', models.CharField(help_text='This is the text that it gets displayed when viewing the email from the email list on your email client.', max_length=255)),
                ('code_name', models.CharField(max_length=255)),
            ],
            options={
                'verbose_name': 'System From Email Address',
                'verbose_name_plural': 'System From Email Address',
            },
        ),
    ]



from django.contrib import admin

from mcmailer.models import SystemFromEmailAddress, EmailNotificationRecipient, EmailTemplate, GmailConnectionToken


@admin.register(SystemFromEmailAddress)
class NotificationEmailAddressAdmin(admin.ModelAdmin):
    list_display = ('email', 'from_official_name',)
    search_fields = ('email', 'from_official_name',)


@admin.register(EmailNotificationRecipient)
class EmailNotificationRecipientAdmin(admin.ModelAdmin):
    list_display = ('email',)
    search_fields = ('email',)


@admin.register(EmailTemplate)
class EmailTemplateAdmin(admin.ModelAdmin):
    readonly_fields = ('html_body_preview',)

    class Media:
        js = (
            'mcmailer/mcmailer_main.js',
        )


@admin.register(GmailConnectionToken)
class GmailConnectionTokenAdmin(admin.ModelAdmin):
    pass



import base64
import datetime
import json
import mimetypes
import os
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

import google.auth.transport.requests
import google.auth.transport.requests
import google.oauth2.credentials
import google.oauth2.credentials
from apiclient import errors, discovery

from mcmailer.models import GmailConnectionToken


def send_message(credentials, sender, to, subject, msg_html, msg_plain, attachment_file=None):
    service = discovery.build('gmail', 'v1', credentials=credentials)
    if attachment_file:
        message1 = create_message_with_attachment(sender, to, subject, msg_html, msg_plain, attachment_file)
    else:
        message1 = create_message_html(sender, to, subject, msg_html, msg_plain)
    result = send_message_internal(service, "me", message1)
    return result


def send_message_internal(service, user_id, message):
    try:
        message = (service.users().messages().send(userId=user_id, body=message).execute())
        print('Message Id: %s' % message['id'])
        return message
    except errors.HttpError as error:
        print('An error occurred: %s' % error)
        return "Error"


def create_message_html(sender, to, subject, msg_html, msg_plain):
    msg = MIMEMultipart('alternative')
    msg['Subject'] = subject
    msg['From'] = sender
    msg['To'] = to
    msg.attach(MIMEText(msg_plain, 'plain'))
    if msg_html:
        msg.attach(MIMEText(msg_html, 'html'))
    return {'raw': base64.urlsafe_b64encode(msg.as_string().encode()).decode()}


def create_message_with_attachment(sender, to, subject, msg_html, msg_plain, attachment_file):
    """Create a message for an email.
    Args:
      sender: Email address of the sender.
      to: Email address of the receiver.
      subject: The subject of the email message.
      msg_html: Html message to be sent
      msg_plain: Alternative plain text message for older email clients
      attachment_file: The path to the file to be attached.

    Returns:
      An object containing a base64url encoded email object.
    """
    message = MIMEMultipart('mixed')
    message['to'] = to
    message['from'] = sender
    message['subject'] = subject

    messageA = MIMEMultipart('alternative')
    messageR = MIMEMultipart('related')

    if msg_html:
        messageR.attach(MIMEText(msg_html, 'html'))
    messageA.attach(MIMEText(msg_plain, 'plain'))
    messageA.attach(messageR)

    message.attach(messageA)

    print("create_message_with_attachment: file: %s" % attachment_file)
    content_type, encoding = mimetypes.guess_type(attachment_file)

    if content_type is None or encoding is not None:
        content_type = 'application/octet-stream'
    main_type, sub_type = content_type.split('/', 1)
    if main_type == 'text':
        fp = open(attachment_file, 'rb')
        msg = MIMEText(fp.read(), _subtype=sub_type)
        fp.close()
    elif main_type == 'image':
        fp = open(attachment_file, 'rb')
        msg = MIMEImage(fp.read(), _subtype=sub_type)
        fp.close()
    elif main_type == 'audio':
        fp = open(attachment_file, 'rb')
        msg = MIMEAudio(fp.read(), _subtype=sub_type)
        fp.close()
    else:
        fp = open(attachment_file, 'rb')
        msg = MIMEBase(main_type, sub_type)
        msg.set_payload(fp.read())
        fp.close()
    filename = os.path.basename(attachment_file)
    msg.add_header('Content-Disposition', 'attachment', filename=filename)
    message.attach(msg)

    return {'raw': base64.urlsafe_b64encode(msg.as_string().encode()).decode()}


def save_credentials(gmail_address, credentials, valid=True):
    try:
        creds = GmailConnectionToken.objects.get(gmail_address=gmail_address)
    except GmailConnectionToken.DoesNotExist:
        creds = GmailConnectionToken()
        creds.gmail_address = gmail_address
    temp = {
        'token': credentials.token,
        'refresh_token': credentials.refresh_token,
        'id_token': credentials.id_token,
        'token_uri': credentials.token_uri,
        'client_id': credentials.client_id,
        'client_secret': credentials.client_secret,
        'scopes': credentials.scopes,
        'expiry': datetime.datetime.strftime(credentials.expiry, '%Y-%m-%d %H:%M:%S')
    }
    creds.json_string = json.dumps(temp)
    creds.valid = valid
    creds.save()
    return temp


def load_credentials(gmail_address, ignore_valid=False):
    try:
        creds = GmailConnectionToken.objects.get(gmail_address=gmail_address)
        # is it valid? do we raise an error?
        if not ignore_valid and not creds.valid:
            raise ValueError('Credentials are not valid.')
        # ok, if we get to here we load/create the Credentials obj()
        temp = json.loads(creds.json_string)
        credentials = google.oauth2.credentials.Credentials(
            temp['token'],
            refresh_token=temp['refresh_token'],
            id_token=temp['id_token'],
            token_uri=temp['token_uri'],
            client_id=temp['client_id'],
            client_secret=temp['client_secret'],
            scopes=temp['scopes'],
        )
        expiry = temp['expiry']
        expiry_datetime = datetime.datetime.strptime(expiry, '%Y-%m-%d %H:%M:%S')
        credentials.expiry = expiry_datetime
        # and now we refresh the token
        # but not if we know that its not a valid token.
        if creds.valid:
            request = google.auth.transport.requests.Request()
            if credentials.expired:
                credentials.refresh(request)
        # and finally, we return this whole deal
        if ignore_valid:
            return True, credentials, creds.valid
        else:
            return True, credentials

    except GmailConnectionToken.DoesNotExist:
        error_message = 'The Gmail address "{}" has not authorized the application to send emails'.format(gmail_address)
        return False, error_message


def send_mc_email(credentials, from_address, to_addresses, from_official_name, subject, msg_plain, msg_html=None,
                  attachment_file=None):
    sender = "{} <{}>".format(from_official_name, from_address)
    to_addresses = ', '.join(to_addresses)
    if attachment_file:
        send_message(credentials, sender, to_addresses, subject, msg_html, msg_plain, attachment_file)
    else:
        send_message(credentials, sender, to_addresses, subject, msg_html, msg_plain)



def get_difference_between_dates(start, end, interval="all"):
    duration = end - start
    duration_in_s = duration.total_seconds()

    def years():
        return divmod(duration_in_s, 31536000)  # Seconds in a year=31536000.

    def days(seconds=None):
        return divmod(seconds if seconds != None else duration_in_s, 86400)  # Seconds in a day = 86400

    def hours(seconds=None):
        return divmod(seconds if seconds != None else duration_in_s, 3600)  # Seconds in an hour = 3600

    def minutes(seconds=None):
        return divmod(seconds if seconds != None else duration_in_s, 60)  # Seconds in a minute = 60

    def seconds(seconds=None):
        if seconds != None:
            return divmod(seconds, 1)
        return duration_in_s

    def total_duration():
        y = years()
        d = days(y[1])  # Use remainder to calculate next variable
        h = hours(d[1])
        m = minutes(h[1])
        s = seconds(m[1])

        return "{} years, {} days, {} hours, {} minutes and {} seconds".format(
            int(y[0]), int(d[0]), int(h[0]), int(m[0]), int(s[0])
        )

    def all_without_seconds():
        y = years()
        d = days(y[1])  # Use remainder to calculate next variable
        h = hours(d[1])
        m = minutes(h[1])

        return "{} years, {} days, {} hours and {} minutes".format(
            int(y[0]), int(d[0]), int(h[0]), int(m[0])
        )

    def year_days_hours():
        y = years()
        d = days(y[1])  # Use remainder to calculate next variable
        h = hours(d[1])

        return "{} years, {} days and {} hours".format(int(y[0]), int(d[0]), int(h[0]))

    # Individual results
    results = {
        'years': int(years()[0]),
        'days': int(days()[0]),
        'hours': int(hours()[0]),
        'minutes': int(minutes()[0]),
        'seconds': int(seconds()),
        'year_days_hours': year_days_hours(),
        'all_without_seconds': all_without_seconds(),
        'all': total_duration(),
    }

    return results[interval]



from django import forms

from mcmailer.models import EmailTemplate


class SendTestEmailForm(forms.Form):
    from_email_address = forms.EmailField()
    from_official_name = forms.CharField(required=False)
    to_email_address = forms.EmailField()
    subject = forms.CharField(required=False)
    body = forms.CharField(widget=forms.Textarea, required=False)
    template_to_use = forms.ModelChoiceField(
        queryset=EmailTemplate.objects.all(),
        empty_label="Nothing",
        help_text='If a template is used, then the From official name, Subject and Body will '
                  'be overridden with the template\'s variables.'
    )



from django.db import models
from django.utils.safestring import mark_safe


class SystemFromEmailAddress(models.Model):
    email = models.EmailField()
    from_official_name = models.CharField(
        max_length=255,
        help_text="This is the text that it gets displayed when "
                  "viewing the email from the email list on your email client."
    )
    code_name = models.CharField(max_length=255)

    class Meta:
        verbose_name = "System From Email Address"
        verbose_name_plural = "System From Email Address"

    def __str__(self):
        return "{}".format(self.email)


class EmailNotificationRecipient(models.Model):
    email = models.EmailField()

    class Meta:
        verbose_name_plural = "Email Notification Recipients"

    def __str__(self):
        return "{}".format(self.email)


class EmailTemplate(models.Model):
    name = models.CharField(max_length=255)
    preview_text = models.CharField(max_length=255)
    subject = models.CharField(max_length=255)
    plain_body = models.TextField()
    html_body = models.TextField()

    def html_body_preview(self):
        if self.html_body:
            return mark_safe('<div id="html_preview"></div>')
        else:
            return 'N/A'

    html_body_preview.short_description = 'HTML preview'
    html_body_preview.allow_tags = True

    def __str__(self):
        return "{}".format(self.name)


class GmailConnectionToken(models.Model):
    json_string = models.TextField()
    valid = models.BooleanField()
    gmail_address = models.EmailField()

    def __str__(self):
        return self.gmail_address



from modeltranslation.translator import register, TranslationOptions

from mcmailer.models import EmailTemplate


@register(EmailTemplate)
class DoorSystemTranslationOptions(TranslationOptions):
    fields = ('subject', 'preview_text', 'plain_body', 'html_body',)



import os

import google_auth_oauthlib.flow
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render

from mcmailer.forms import SendTestEmailForm
from mcmailer.models import SystemFromEmailAddress
from mcmailer.utils.sendgmail import save_credentials, load_credentials, send_mc_email

API_SCOPE = ['https://www.googleapis.com/auth/gmail.send', ]


def get_error_msg(var_name):
    return 'Requested setting {0}, but settings are not configured. ' \
           'You must define the variable {0} in your settings.py'.format(var_name)


try:
    JSON_PATH = settings.JSON_PATH
except AttributeError:
    raise ImproperlyConfigured(get_error_msg('JSON_PATH'))
try:
    LOCAL_HOST = settings.DEBUG
except AttributeError:
    raise ImproperlyConfigured(get_error_msg('LOCAL_HOST'))

if LOCAL_HOST:
    try:
        REDIRECT_URI = settings.LOCAL_HOST_REDIRECT_URI
    except AttributeError:
        raise ImproperlyConfigured(get_error_msg('LOCAL_HOST_REDIRECT_URI'))
    # we don't have ssl on local host
    os.environ['OAUTHLIB_INSECURE_TRANSPORT'] = '1'
else:
    try:
        REDIRECT_URL = settings.PRODUCTION_REDIRECT_URI
    except AttributeError:
        raise ImproperlyConfigured(get_error_msg('PRODUCTION_REDIRECT_URI'))

GMAIL_SESSION_NAME = 'gmail_session_name'


def save_email_address_to_tmp_file(gmail_address):
    with open(os.path.join(os.getcwd(), "gmail_address.tmp"), "w") as tmp_file:
        tmp_file.write(gmail_address)


def get_email_address_from_tmp_file():
    tmp_file_path = os.path.join(os.getcwd(), "gmail_address.tmp")
    with open(tmp_file_path, "r") as tmp_file:
        gmail_address = tmp_file.read()
    os.remove(tmp_file_path)
    return gmail_address


@login_required
def connect_gmail(request):
    if request.method == "POST":
        gmail_address = request.POST.get('email')
        save_email_address_to_tmp_file(gmail_address)
        flow = google_auth_oauthlib.flow.Flow.from_client_secrets_file(JSON_PATH, scopes=API_SCOPE)
        flow.redirect_uri = REDIRECT_URI
        authorization_url, state = flow.authorization_url(
            access_type='offline',
            login_hint=gmail_address,
            include_granted_scopes='true'
        )
        return HttpResponseRedirect(authorization_url)
    context = {
        'email_addresses': SystemFromEmailAddress.objects.all()
    }
    return render(request, 'auth_gmail_address.html', context)


@login_required
def g_auth_endpoint(request):
    gmail_address = get_email_address_from_tmp_file()
    state = request.GET.get('state', None)
    flow = google_auth_oauthlib.flow.Flow.from_client_secrets_file(JSON_PATH, scopes=API_SCOPE, state=state)
    flow.redirect_uri = REDIRECT_URI
    authorization_response = request.build_absolute_uri()
    flow.fetch_token(authorization_response=authorization_response)
    credentials = flow.credentials
    temp = save_credentials(gmail_address, credentials)
    return HttpResponse("Successful authorization of the gmail address {}".format(gmail_address))


def parse_template(text, objects):
    """
    Loop through the object's attributes and find all the variables in the 'text' that have
    the name of the attribute surrounded with the prefix and suffix
    and replace them with the value of the object's attribute.

    :param text: A str to replace the variables found from the objects attributes.
    :param objects: A list of objects
    :return: str the parsed text.
    """
    special_variable_prefix = '(%%'
    special_variable_suffix = '%%)'
    for obj in objects:
        for attr in dir(obj):
            template_variable = "{}{}{}".format(special_variable_prefix, attr, special_variable_suffix)
            if template_variable in text:
                text = text.replace(template_variable, getattr(obj, attr))
    return text


def send_test_email(request):
    if request.method == 'POST':
        form = SendTestEmailForm(request.POST)
        if form.is_valid():
            # Send the email
            system_from_email_address = SystemFromEmailAddress.objects.first()

            from_address = form.cleaned_data['from_email_address']
            success, credentials = load_credentials(from_address)

            to_address = form.cleaned_data['to_email_address']

            if form.cleaned_data['template_to_use']:
                email_template = form.cleaned_data['template_to_use']
                from_official_name = parse_template(
                    text=email_template.from_official_name,
                    objects=[system_from_email_address]
                )
                subject = parse_template(
                    text=email_template.subject,
                    objects=[system_from_email_address]
                )
                body = parse_template(
                    text=email_template.html_body,
                    objects=[system_from_email_address]
                )
            else:
                from_official_name = form.cleaned_data['from_official_name']
                subject = form.cleaned_data['subject']
                body = form.cleaned_data['body']

            send_mc_email(
                credentials,
                from_address=from_address,
                to_addresses=[to_address],
                from_official_name=from_official_name,
                subject=subject,
                msg_plain=body,
                msg_html=body
            )
            return HttpResponse('Email successfully sent!')
    else:
        form = SendTestEmailForm()
        context = {
            'send_test_email_form': form
        }
        return render(request, 'send_test_email.html', context)
